@@ -4,65 +4,30 @@
 ddml_att <- function(y, D, X,
                      learners,
                      learners_DX = learners,
-                     sample_folds = 2,
+                     sample_folds = 10,
                      ensemble_type = "nnls",
                      shortstack = FALSE,
-                     cv_folds = 5,
+                     cv_folds = 10,
                      custom_ensemble_weights = NULL,
                      custom_ensemble_weights_DX = custom_ensemble_weights,
-                     subsamples_D0 = NULL,
-                     subsamples_D1 = NULL,
-                     cv_subsamples_list_D0 = NULL,
-                     cv_subsamples_list_D1 = NULL,
+                     cluster_variable = seq_along(y),
+                     subsamples_byD = NULL,
+                     cv_subsamples_byD = NULL,
                      trim = 0.01,
                      silent = FALSE) {
   # Data parameters
   nobs <- length(y)
   is_D0 <- which(D == 0)
-  nobs_D0 <- length(is_D0)
-  nobs_D1 <- nobs - nobs_D0
 
-  # Create sample fold tuple by treatment
-  if (is.null(subsamples_D0) | is.null(subsamples_D1)) {
-    subsamples_D0 <- generate_subsamples(nobs_D0, sample_folds)
-    subsamples_D1 <- generate_subsamples(nobs_D1, sample_folds)
-  }#IF
-  sample_folds <- length(subsamples_D0)
+  # Create sample and cv-fold tuples
+  cf_indxs <- get_crossfit_indices(cluster_variable = cluster_variable, D = D,
+                                   sample_folds = sample_folds,
+                                   cv_folds = cv_folds,
+                                   subsamples_byD = subsamples_byD,
+                                   cv_subsamples_byD = cv_subsamples_byD)
 
-  # Create cv-subsamples tuple by treatment
-  if (is.null(cv_subsamples_list_D0) | is.null(cv_subsamples_list_D1)) {
-    cv_subsamples_list_D0 <- rep(list(NULL), sample_folds)
-    cv_subsamples_list_D1 <- rep(list(NULL), sample_folds)
-    for (k in 1:sample_folds) {
-      nobs_D0_k <- nobs_D0 - length(subsamples_D0[[k]])
-      nobs_D1_k <- nobs_D1 - length(subsamples_D1[[k]])
-      cv_subsamples_list_D0[[k]] <- generate_subsamples(nobs_D0_k, cv_folds)
-      cv_subsamples_list_D1[[k]] <- generate_subsamples(nobs_D1_k, cv_folds)
-    }# FOR
-  }#IF
-
-  # Merge subsamples across treatment and create auxilliary control matrix
-  subsamples <- subsamples_D0
-  cv_subsamples_list <- cv_subsamples_list_D0
-  auxilliary_X_D0 <- rep(list(NULL), sample_folds)
-  auxilliary_X_D1 <- rep(list(NULL), sample_folds)
-  for (k in 1:sample_folds) {
-    # Sample folds
-    subsamples[[k]] <- sort(c((1:nobs)[is_D0][subsamples_D0[[k]]],
-                              (1:nobs)[-is_D0][subsamples_D1[[k]]]))
-    # CV folds
-    nobs_k <- nobs - length(subsamples[[k]])
-    is_D0_k <- which(D[-subsamples[[k]]] == 0)
-    is_D1_k <- which(D[-subsamples[[k]]] == 1)
-    for (j in 1:cv_folds) {
-      indx_D0 <- is_D0_k[cv_subsamples_list_D0[[k]][[j]]]
-      indx_D1 <- is_D1_k[cv_subsamples_list_D1[[k]][[j]]]
-      cv_subsamples_list[[k]][[j]] <- sort(c(indx_D0, indx_D1))
-    }#FOR
-
-    # Auxilliary X (only need treated observations)
-    auxilliary_X_D1[[k]] <- X[-is_D0, , drop=F][subsamples_D1[[k]], , drop=F]
-  }#FOR
+  # Create tuple for extrapolated fitted values
+  aux_indxs <- get_auxiliary_indx(cf_indxs$subsamples_byD, D)
 
   # Print to progress to console
   if (!silent) cat("DDML estimation in progress. \n")
@@ -72,18 +37,18 @@ ddml_att <- function(y, D, X,
                         learners = learners, ensemble_type = ensemble_type,
                         shortstack = shortstack,
                         custom_ensemble_weights = custom_ensemble_weights,
-                        cv_subsamples_list = cv_subsamples_list_D0,
-                        subsamples = subsamples_D0,
+                        subsamples = cf_indxs$subsamples_byD[[1]],
+                        cv_subsamples_list = cf_indxs$cv_subsamples_byD[[1]],
                         silent = silent, progress = "E[Y|D=0,X]: ",
-                        auxilliary_X = auxilliary_X_D1)
+                        auxiliary_X = get_auxiliary_X(aux_indxs[[1]], X))
 
   # Compute estimates of E[D|X]
   D_X_res <- get_CEF(D, X,
                      learners = learners_DX, ensemble_type = ensemble_type,
                      shortstack = shortstack,
                      custom_ensemble_weights = custom_ensemble_weights_DX,
-                     cv_subsamples_list = cv_subsamples_list,
-                     subsamples = subsamples,
+                     subsamples = cf_indxs$subsamples,
+                     cv_subsamples_list = cf_indxs$cv_subsamples_list,
                      silent = silent, progress = "E[D|X]: ")
 
   # Compute estimates of E[D] -- simple computation of averages here
@@ -92,7 +57,7 @@ ddml_att <- function(y, D, X,
                    ensemble_type = "average",
                    shortstack = FALSE,
                    cv_subsamples_list = NULL,
-                   subsamples = subsamples,
+                   subsamples = cf_indxs$subsamples,
                    silent = TRUE)
 
   # Update ensemble type to account for (optional) custom weights
@@ -103,17 +68,9 @@ ddml_att <- function(y, D, X,
   multiple_ensembles <- nensb > 1
 
   # Construct reduced form variables
-  g_D0 <- matrix(0, nobs, nensb)
-  g_D0[is_D0, ] <- y_X_D0_res$oos_fitted
-  if (!multiple_ensembles) {
-    for (k in 1:sample_folds) {
-      g_D0[-is_D0][subsamples_D1[[k]]] <- y_X_D0_res$auxilliary_fitted[[k]]
-    }#FOR
-  } else {
-    for (k in 1:sample_folds) {
-      g_D0[-is_D0, ][subsamples_D1[[k]], ] <- y_X_D0_res$auxilliary_fitted[[k]]
-    }#FOR
-  }#IF
+  g_X_D0<- extrapolate_CEF(D = D,
+                             CEF_res_byD = list(list(fit = y_X_D0_res, d = 0)),
+                             aux_indxs = aux_indxs)[, , 1]
   m_X <- D_X_res$oos_fitted
 
   # Trim propensity scores, return warnings
@@ -123,8 +80,8 @@ ddml_att <- function(y, D, X,
   y_copy <- matrix(rep(y, nensb), nobs, nensb)
   D_copy <- matrix(rep(D, nensb), nobs, nensb)
   p_copy <- matrix(rep(D_res$oos_fitted, nensb), nobs, nensb)
-  psi_b <- D_copy * (y_copy - g_D0) / p_copy -
-    m_X_tr * (1 - D_copy) * (y_copy - g_D0) / (p_copy * (1 - m_X_tr))
+  psi_b <- D_copy * (y_copy - g_X_D0) / p_copy -
+    m_X_tr * (1 - D_copy) * (y_copy - g_X_D0) / (p_copy * (1 - m_X_tr))
   psi_a <- -D_copy / p_copy
   att <- -colMeans(psi_b) / colMeans(psi_a)
   names(att) <- ensemble_type
@@ -138,7 +95,7 @@ ddml_att <- function(y, D, X,
                D_X = D_X_res$mspe)
 
   # Organize reduced form predicted values
-  oos_pred <- list(EY_D0_X = g_D0, ED_X = m_X, ED = D_res$oos_fitted)
+  oos_pred <- list(EY_D0_X = g_X_D0, ED_X = m_X, ED = D_res$oos_fitted)
 
   # Organize output
   ddml_fit <- list(att = att, weights = weights, mspe = mspe,
@@ -146,10 +103,9 @@ ddml_att <- function(y, D, X,
                    oos_pred = oos_pred,
                    learners = learners,
                    learners_DX = learners_DX,
-                   subsamples_D0 = subsamples_D0,
-                   subsamples_D1 = subsamples_D1,
-                   cv_subsamples_list_D0 = cv_subsamples_list_D0,
-                   cv_subsamples_list_D1 = cv_subsamples_list_D1,
+                   cluster_variable = cluster_variable,
+                   subsamples = cf_indxs$subsamples_byD,
+                   cv_subsamples_list = cf_indxs$cv_subsamples_byD,
                    ensemble_type = ensemble_type)
 
   # Print estimation progress
@@ -172,7 +128,9 @@ summary.ddml_att <- function(object, ...) {
                                                    psi_a = object$psi_a,
                                                    psi_b = object$psi_b,
                                                    ensemble_type =
-                                                     object$ensemble_type)
+                                                     object$ensemble_type,
+                                                   cluster_variable =
+                                                     object$cluster_variable)
   class(coefficients) <- c("summary.ddml_att", class(coefficients))
   coefficients
 }#SUMMARY.DDML_ATT
