@@ -10,7 +10,7 @@ get_CEF <- function(y, X, Z = NULL,
                     cv_subsamples_list,
                     silent = FALSE,
                     progress = NULL,
-                    auxilliary_X = NULL,
+                    auxiliary_X = NULL,
                     shortstack_y = y) {
   # Compute CEF
   if (shortstack) {
@@ -22,7 +22,7 @@ get_CEF <- function(y, X, Z = NULL,
                            compute_insample_predictions,
                          subsamples = subsamples,
                          silent = silent, progress = progress,
-                         auxilliary_X = auxilliary_X,
+                         auxiliary_X = auxiliary_X,
                          shortstack_y = shortstack_y)
   } else {
     res <- crosspred(y, X, Z,
@@ -36,7 +36,7 @@ get_CEF <- function(y, X, Z = NULL,
                      subsamples = subsamples,
                      cv_subsamples_list = cv_subsamples_list,
                      silent = silent, progress = progress,
-                     auxilliary_X = auxilliary_X)
+                     auxiliary_X = auxiliary_X)
   }#IFELSE
   update_progress(silent)
 
@@ -48,3 +48,28 @@ get_CEF <- function(y, X, Z = NULL,
 update_progress <- function(silent) {
   if (!silent) cat(" -- Done! \n")
 }#UPDATE_PROGRESS
+
+# Construct CEF from auxiliary_X
+extrapolate_CEF <- function(D, CEF_res_byD, aux_indxs) {
+  # Data parameters
+  nCEF <- length(CEF_res_byD)
+  nobs <- length(D)
+  D_levels <- lapply(CEF_res_byD, function(x) x$d)
+  is_D <- rep(list(NULL), nCEF)
+  for (d in 1:nCEF) is_D[[d]] <- which(D == D_levels[d])
+  nensb <- ncol(as.matrix(CEF_res_byD[[1]][[1]]$oos_fitted))
+  sample_folds <- length(CEF_res_byD[[1]][[1]]$auxiliary_fitted)
+
+  # Populate CEF
+  g_X_byD <- array(0, dim = c(nobs, nensb, nCEF))
+  for (d in 1:nCEF) {
+    g_X_byD[is_D[[d]], , d] <- CEF_res_byD[[d]][[1]]$oos_fitted
+    for (k in 1:sample_folds) {
+      g_X_byD[aux_indxs[[d]][[k]], , d] <-
+        CEF_res_byD[[d]][[1]]$auxiliary_fitted[[k]]
+    }#FOR
+  }#FOR
+
+  # return as array, third dimension is different levels of d
+  g_X_byD
+}#EXTRAPOLATE_CEF
