@@ -16,6 +16,31 @@ test_that("ddml_fpliv computes with a single model", {
   expect_equal(length(ddml_fpliv_fit$coef), 1)
 })#TEST_THAT
 
+test_that("ddml_fpliv computes with a single model and dependence", {
+  # Simulate small dataset
+  n_cluster <- 250
+  nobs <- 500
+  X <- cbind(1, matrix(rnorm(n_cluster*39), n_cluster, 39))
+  Z_tld <-  X %*% runif(40) + rnorm(n_cluster)
+  fun <- stepfun(quantile(Z_tld, probs = 0.5), c(0, 1))
+  Z <- fun(Z_tld)
+  cluster_variable <- sample(1:n_cluster, nobs, replace = TRUE)
+  Z <- as.matrix(Z[cluster_variable, drop = F])
+  X <- X[cluster_variable, , drop = F]
+  eps <- rnorm(nobs)
+  D <- Z + X %*% runif(40) + eps
+  y <- D + X %*% runif(40) + 0.1 * eps + rnorm(nobs)
+  # Define arguments
+  learners <- list(list(fun = ols))
+  ddml_fpliv_fit <- ddml_fpliv(y, D, Z, X,
+                               learners,
+                               cluster_variable = cluster_variable,
+                               sample_folds = 3,
+                               silent = T)
+  # Check output with expectations
+  expect_equal(length(ddml_fpliv_fit$coef), 1)
+})#TEST_THAT
+
 test_that("ddml_fpliv computes with an ensemble procedure", {
   # Simulate small dataset
   nobs <- 200
