@@ -104,3 +104,32 @@ test_that("prediction w/ ensembles returns fitted values w/ custom weights", {
   # Check output with expectations
   expect_equal(dim(ens_fitted), c(length(D), 5))
 })#TEST_THAT
+
+test_that("ensemble returns mean_y for constant outcomes", {
+  # Simulate dataset with constant y
+  X <- cbind(1, matrix(rnorm(100*39), 100, 39))
+  Z <- matrix(rnorm(100*10), 100, 10)
+  y <- rep(42, 100)  # Constant outcome
+
+  # Define arguments
+  learners <- list(list(fun = mdl_glmnet),
+                   list(fun = ols))
+
+  # Compute ensemble and expect warning
+  expect_warning({
+    ens_fit <- ensemble(y, X, Z,
+                        type = c("average", "ols", "singlebest"),
+                        learners = learners,
+                        cv_folds = 3,
+                        silent = T)
+  }, "Outcome variable y is constant")
+
+  # Check predictions
+  ens_fitted <- predict.ensemble(ens_fit, newdata = X, newZ = Z)
+
+  # Check output matches expectations
+  expect_equal(dim(ens_fitted), c(nrow(X), length(learners)))
+  expect_true(all(ens_fitted == 42))
+  expect_true(ens_fit$constant_y)
+  expect_null(ens_fit$mdl_fits)
+})
